/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.util.activator.impl;

import static com.openexchange.util.custom.base.NullUtil.notNull;
import java.lang.reflect.Constructor;
import java.util.Dictionary;
import java.util.Set;


import javax.annotation.concurrent.ThreadSafe;
import org.osgi.framework.BundleContext;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.openexchange.annotation.Nullable;
import com.openexchange.osgi.ServiceListing;
import com.openexchange.osgi.ServiceSet;
import com.openexchange.util.activator.ServiceDependencyResolver;
import com.openexchange.util.activator.ServiceRegistrationHandler;

/**
 * This abstract service tracker can wait for some service and register another one if all become available.
 *
 * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
 * @author <a href="mailto:pascal.bleser@open-xchange.com">Pascal Bleser</a>
 */
@ThreadSafe
public class ServiceAwareDependentServiceRegisterer<S, T extends S> extends InstanceFactoryServiceAwareDependentServiceRegistererTemplate<S, T> {

    private final @Nullable ServiceRegistrationHandler<T> serviceRegistrationHandler;        
    private final Constructor<T> constructor;
    
    public ServiceAwareDependentServiceRegisterer(final BundleContext context,
        final ImmutableSet<Class<?>> serviceTypes,
        final Class<T> serviceInstanceClass,
        final Dictionary<String, ?> properties,
        final Set<MandatoryServiceDependency<?>> mandatoryServices,
        final Set<OptionalServiceDependency<?>> optionalServices,
        final ImmutableMap<Class<?>, ServiceSet<?>> serviceSets,
        final ImmutableMap<Class<?>, ServiceListing<?>> serviceListings,
        final Set<String> propertiesOfInterest,
        final Constructor<T> constructor,
        final @Nullable ServiceRegistrationHandler<T> serviceRegistrationHandler) {
        super(context, serviceTypes, notNull(serviceInstanceClass.getName()),
            properties,
            mandatoryServices,
            optionalServices,
            serviceSets,
            serviceListings,
            propertiesOfInterest
        );
        this.serviceRegistrationHandler = serviceRegistrationHandler;
        this.constructor = constructor;
    }
    
    @Override
    protected T createInstance(final ServiceDependencyResolver resolver) throws Exception {
        return RegistrationTools.createInstance(
            serviceInstanceName,
            constructor,
            serviceTypes,
            resolver,
            serviceRegistrationHandler
        );
    }
}
