/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.util.custom.i18n;

import java.util.Objects;


import com.google.common.base.MoreObjects;
import com.google.common.collect.ImmutableMap;
import com.openexchange.annotation.Nullable;
import com.openexchange.exception.OXException;

/**
 * {@link ValidPropertyTranslation}
 *
 * @author <a href="mailto:pascal.bleser@open-xchange.com">Pascal Bleser</a>
 * @since v7.8.2
 */
public class ValidPropertyTranslation implements PropertyTranslation {
    private final String name;
    private final ImmutableMap<String, String> map;
    private final String fallback;
    public ValidPropertyTranslation(final String name,
        final ImmutableMap<String, String> map,
        final String fallback) {
        this.name = name;
        this.map = map;
        this.fallback = fallback;
    }
    @Override
    public String name() {
        return name;
    }
    @Override
    public boolean isValid() {
        return true;
    }
    @Override
    public String get(final @Nullable String lang) throws OXException {
        if (lang == null) {
            return fallback;
        }
        if (map.containsKey(lang)) {
            final String value = map.get(lang);
            if (value != null) {
                return value;
            }
        }
        return fallback;
    }
    @Override
    public int hashCode() {
        return Objects.hash(name, fallback, map);
    }
    @Override
    public boolean equals(@Nullable Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final ValidPropertyTranslation other = (ValidPropertyTranslation) obj;
        return Objects.equals(name, other.name)
            && Objects.equals(fallback, other.fallback)
            && Objects.equals(map, other.map)
        ;
    }
   @Override
    public @Nullable String toString() {
        return MoreObjects.toStringHelper(this)
        .add("name", name)
        .add("fallback", fallback)
        .add("map", map)
        .toString();
    }
}